@@ -13,14 +13,13 @@ typedef guint16 WORD ;
 typedef guint32 LONG ;
 typedef guint64 DLONG ;
 
-/* EXTREMELY IMPORTANT TO PASS A BYTE PTR ! */
-#define BIFF_GETBYTE(p) (*(p+0))
-#define BIFF_GETWORD(p) (*(p+0)+(*(p+1)<<8))
-#define BIFF_GETLONG(p) (*(p+0)+ \
-		        (*(p+1)<<8)+ \
-		        (*(p+2)<<16)+ \
-		        (*(p+3)<<24))
-#define BIFF_GETDLONG(p) (DLONG)(BIFF_GETLONG(p)+(((DLONG)BIFF_GETLONG(p+4))<<32))
+#define BIFF_GETBYTE(p) (*((const BYTE *)(p)+0))
+#define BIFF_GETWORD(p) (*((const BYTE *)(p)+0) | (*((const BYTE *)(p)+1)<<8))
+#define BIFF_GETLONG(p) (*((const BYTE *)(p)+0) | \
+                        (*((const BYTE *)(p)+1)<<8) | \
+                        (*((const BYTE *)(p)+2)<<16) | \
+                        (*((const BYTE *)(p)+3)<<24))
+#define BIFF_GETDLONG(p) (BIFF_GETLONG(p) | (((DLONG)BIFF_GETLONG((const BYTE *)(p)+4))<<32))
 
 double biff_getdouble(guint8 *p);
 	
@@ -55,4 +54,6 @@ extern BIFF_QUERY *ms_biff_query_copy (const BIFF_QUERY *p) ;
  **/
 extern int ms_biff_query_next (BIFF_QUERY *) ;
 extern void ms_biff_query_destroy (BIFF_QUERY *) ;
+/* Returns a stream which contains the data in the BIFF record. */
+extern MS_OLE_STREAM *ms_biff_query_data_to_stream (BIFF_QUERY *);
 #endif
