@@ -24,12 +24,15 @@
 #include "color.h"
 #include "sheet-object.h"
 #include "style.h"
+#include "main.h"
 
 #include "ms-ole.h"
 #include "ms-biff.h"
 #include "ms-formula.h"
 #include "ms-excel.h"
 #include "ms-excel-biff.h"
+#include "ms-obj.h"
+#include "ms-escher.h"
 
 #define EXCEL_DEBUG 0
 
@@ -101,7 +104,7 @@ biff_string_get_flags (BYTE *ptr,
 char *
 biff_get_text (BYTE *pos, guint32 length, guint32* byte_length)
 {
-	int lp ;
+	guint32 lp ;
 	char *ans;
 	BYTE *ptr ;
 	guint32 byte_len ;
@@ -117,7 +120,7 @@ biff_get_text (BYTE *pos, guint32 length, guint32* byte_length)
 	if (!length)
 		return 0 ;
 
-	ans = (char *) g_malloc (sizeof (char) * length + 2);
+	ans = (char *) g_new (char, length + 2);
 
 	header = biff_string_get_flags (pos,
 					&high_byte,
@@ -157,8 +160,15 @@ biff_get_text (BYTE *pos, guint32 length, guint32* byte_length)
 
 	for (lp = 0; lp < length; lp++)
 	{
-		ans [lp] = (char) *ptr ;
-		ptr += high_byte ? 2 : 1;
+		guint16 c;
+		if (high_byte) {
+			c = BIFF_GETWORD(ptr);
+			ptr+=2;
+		} else {
+			c = BIFF_GETBYTE(ptr);
+			ptr+=1;
+		}
+		ans[lp] = (char)c;
 	}
 	ans[lp] = 0;
 	return ans;
@@ -175,8 +185,8 @@ biff_get_global_string(MS_EXCEL_SHEET *sheet, int number)
 	return wb->global_strings[number] ;
 }
 
-char *
-biff_get_error_text (guint8 err)
+const char *
+biff_get_error_text (const guint8 err)
 {
 	char *buf ;
 	switch (err)
@@ -376,12 +386,12 @@ biff_boundsheet_data_new (MS_EXCEL_WORKBOOK *wb, BIFF_QUERY * q, eBiff_version v
 		break;
 	}
 	if (ver == eBiffV8) {
-		int strlen = BIFF_GETWORD (q->data + 6);
-		ans->name = biff_get_text (q->data + 8, strlen, NULL);
+		int slen = BIFF_GETWORD (q->data + 6);
+		ans->name = biff_get_text (q->data + 8, slen, NULL);
 	} else {
-		int strlen = BIFF_GETBYTE (q->data + 6);
+		int slen = BIFF_GETBYTE (q->data + 6);
 
-		ans->name = biff_get_text (q->data + 7, strlen, NULL);
+		ans->name = biff_get_text (q->data + 7, slen, NULL);
 	}
 
 	/*
@@ -549,9 +559,9 @@ static gboolean
 biff_font_data_destroy (gpointer key, BIFF_FONT_DATA *fd, gpointer userdata)
 {
 	g_free (fd->fontname) ;
-	g_free (fd) ;
 	if (fd->style_font)
 		style_font_unref (fd->style_font) ;
+	g_free (fd) ;
 	return 1 ;
 }
 
@@ -715,20 +725,27 @@ ms_excel_palette_new (BIFF_QUERY * q)
 static StyleColor *
 ms_excel_palette_get (MS_EXCEL_PALETTE *pal, guint idx)
 {
+	if (!pal) return NULL ;
 	if (idx < pal->length)
 	{
 		if (pal->gnum_cols[idx] == NULL) {
+			gushort r, g, b;
+			r = pal->red[idx]*(65536/128-1);
+			g = pal->green[idx]*(65536/128-1);
+			b = pal->blue[idx]*(65536/128-1);
 			if (EXCEL_DEBUG>4)
-				printf ("New color in slot %d : RGB= %d,%d,%d\n",
-					idx, pal->red[idx], pal->green[idx], pal->blue[idx]) ;
-			pal->gnum_cols[idx] = style_color_new (pal->red[idx], pal->green[idx],
-							       pal->blue[idx]) ;
-			g_return_val_if_fail (pal->gnum_cols[idx], NULL) ;
+				printf ("New color in slot %d : RGB= %d,%d,%d -> %d,%d,%d\n",
+					idx, pal->red[idx], pal->green[idx], pal->blue[idx],
+					r, g, b);
+			pal->gnum_cols[idx] = style_color_new (r, g, b);
+			g_return_val_if_fail (pal->gnum_cols[idx], NULL);
 		}
-		return pal->gnum_cols[idx] ;
+		return pal->gnum_cols[idx];
 	}
+	else if (idx == 64)
+		return style_color_new (65535, 65535, 65535);
 	else
-		return 0 ;
+		return 0;
 }
 
 static void
@@ -771,43 +788,56 @@ typedef struct _BIFF_XF_DATA {
 	eBiff_border_orientation border_orientation;
 	StyleBorderType border_linestyle;
 	BYTE fill_pattern_idx;
-	BYTE foregnd_col;
-	BYTE backgnd_col;
+	BYTE pat_foregnd_col;
+	BYTE pat_backgnd_col;
 } BIFF_XF_DATA;
 
 static void
-ms_excel_set_cell_colors (MS_EXCEL_SHEET * sheet, Cell * cell, BIFF_XF_DATA * xf)
+ms_excel_set_cell_colors (MS_EXCEL_SHEET * sheet, Cell * cell, BIFF_XF_DATA * xf, StyleColor *basefore)
 {
 	MS_EXCEL_PALETTE *p = sheet->wb->palette;
-	StyleColor *fore, *back ;
+	StyleColor *fore, *back;
 	int col;
 
 	if (!p || !xf) {
 		if (EXCEL_DEBUG>0)
-			printf ("Internal Error: No palette !\n") ;
+			printf ("Internal Error: No palette !\n");
 		return;
 	}
 
-	fore = ms_excel_palette_get (sheet->wb->palette, xf->foregnd_col) ;
-	back = ms_excel_palette_get (sheet->wb->palette, xf->backgnd_col) ;
-	g_return_if_fail (fore) ;
-	g_return_if_fail (back) ;
-	cell_set_color_from_style (cell, fore, back) ;
+	/* FIXME: gnumeric needs better colour support */
+	/* Whack into cell.c(cell_draw):
+	   printf ("Background %p (%d %d %d)\n", style->back_color, style->back_color->color.red,
+	   style->back_color->color.green, style->back_color->color.blue) ; */
+
+	if (!basefore) {
+		fore = ms_excel_palette_get (sheet->wb->palette, xf->pat_foregnd_col);
+		back = ms_excel_palette_get (sheet->wb->palette, xf->pat_backgnd_col);
+	} else {
+		fore = basefore;
+		back = ms_excel_palette_get (sheet->wb->palette, xf->pat_foregnd_col);
+	}
+	if (fore && back)
+		cell_set_color_from_style (cell, fore, back);
+	else if (EXCEL_DEBUG>0)
+		printf ("Missing color\n");
 }
 
 /**
  * Search for a font record from its index in the workbooks font table
  * NB. index 4 is omitted supposedly for backwards compatiblity
+ * Returns the font color if there is one.
  **/
-static void
+static StyleColor *
 ms_excel_set_cell_font (MS_EXCEL_SHEET * sheet, Cell * cell, BIFF_XF_DATA * xf)
 {
 	BIFF_FONT_DATA *fd = g_hash_table_lookup (sheet->wb->font_data, &xf->font_idx) ;
+	StyleColor *col ;
 
 	if (!fd)
 	{
 		printf ("Unknown fount idx %d\n", xf->font_idx);
-		return ;
+		return NULL ;
 	}
 	g_assert (fd->index != 4);
 
@@ -818,14 +848,20 @@ ms_excel_set_cell_font (MS_EXCEL_SHEET * sheet, Cell * cell, BIFF_XF_DATA * xf)
 		cell_set_font_from_style (cell, fd->style_font) ;
 	else
 		printf ("Duff StyleFont\n") ;
+	return ms_excel_palette_get (sheet->wb->palette, fd->color_idx) ;
 }
 
 static void
 ms_excel_set_cell_xf (MS_EXCEL_SHEET * sheet, Cell * cell, guint16 xfidx)
 {
 	BIFF_XF_DATA *xf;
+	StyleColor *fore;
 
-	g_return_if_fail (cell->value) ;
+	if (!cell->value) {
+		if (EXCEL_DEBUG>0)
+			printf ("FIXME: Error setting xf on cell\n");
+		return;
+	}
 
 	if (xfidx == 0){
 /*		printf ("Normal cell formatting\n"); */
@@ -855,7 +891,7 @@ ms_excel_set_cell_xf (MS_EXCEL_SHEET * sheet, Cell * cell, guint16 xfidx)
 	 * Well set it up then ! FIXME: hack ! 
 	 */
 	cell_set_alignment (cell, xf->halign, xf->valign, ORIENT_HORIZ, 0);
-	ms_excel_set_cell_font (sheet, cell, xf);
+	fore = ms_excel_set_cell_font (sheet, cell, xf);
 	if (sheet->wb->palette)
 	{
 		int lp;
@@ -875,7 +911,7 @@ ms_excel_set_cell_xf (MS_EXCEL_SHEET * sheet, Cell * cell, guint16 xfidx)
 				cell_set_format_from_style (cell, xf->style_format) ;
 		}
 	}
-	ms_excel_set_cell_colors (sheet, cell, xf);
+	ms_excel_set_cell_colors (sheet, cell, xf, fore);
 }
 
 static StyleBorderType
@@ -1082,12 +1118,12 @@ biff_xf_data_new (MS_EXCEL_WORKBOOK *wb, BIFF_QUERY * q, eBiff_version ver)
 		xf->fill_pattern_idx = biff_xf_map_border ((data & 0xfc000000) >> 26);
 
 		data = BIFF_GETWORD (q->data + 18);
-		xf->foregnd_col = (data & 0x007f);
-		xf->backgnd_col = (data & 0x3f80) >> 7;
-	} else {
+		xf->pat_foregnd_col = (data & 0x007f);
+		xf->pat_backgnd_col = (data & 0x3f80) >> 7;
+	} else { /* Biff 7 */
 		data = BIFF_GETWORD (q->data + 8);
-		xf->foregnd_col = (data & 0x007f);
-		xf->backgnd_col = (data & 0x1f80) >> 7;
+		xf->pat_foregnd_col = (data & 0x007f);
+		xf->pat_backgnd_col = (data & 0x1f80) >> 7;
 
 		data = BIFF_GETWORD (q->data + 10);
 		xf->fill_pattern_idx = data & 0x03f;
@@ -1126,6 +1162,8 @@ biff_xf_data_new (MS_EXCEL_WORKBOOK *wb, BIFF_QUERY * q, eBiff_version ver)
 		/*	        printf ("Inserting into style XF hash with : %d\n", xf->index) ; */
 		g_hash_table_insert (wb->XF_style_records, &xf->index, xf) ;
 	}
+	if (EXCEL_DEBUG>0)
+		printf ("XF %d Fore %d, Back %d\n", xf->index, xf->pat_foregnd_col, xf->pat_backgnd_col) ;
 }
 
 static gboolean 
@@ -1164,9 +1202,10 @@ ms_excel_sheet_shared_formula (MS_EXCEL_SHEET *sheet,
 	sf = g_hash_table_lookup (sheet->shared_formulae, &k) ;
 	if (sf)
 		return ms_excel_parse_formula (sheet, sf->data,
-					       col, row, shr_col, shr_row,
+					       col, row, 1,
 					       sf->data_len) ;
-	printf ("Duff shared formula index %d %d\n", col, row) ;
+	if (EXCEL_DEBUG>0)
+		printf ("Duff shared formula index %d %d\n", col, row) ;
 	return strdup ("00") ;
 }
 
@@ -1177,7 +1216,8 @@ ms_excel_sheet_set_version (MS_EXCEL_SHEET *sheet, eBiff_version ver)
 }
 
 void
-ms_excel_sheet_insert (MS_EXCEL_SHEET * sheet, int xfidx, int col, int row, char *text)
+ms_excel_sheet_insert (MS_EXCEL_SHEET * sheet, int xfidx,
+		       int col, int row, const char *text)
 {
 	Cell *cell = sheet_cell_fetch (sheet->gnum_sheet, col, row);
 	/* NB. cell_set_text _certainly_ strdups *text */
@@ -1191,6 +1231,33 @@ ms_excel_sheet_insert (MS_EXCEL_SHEET * sheet, int xfidx, int col, int row, char
 	ms_excel_set_cell_xf (sheet, cell, xfidx);
 }
 
+static void
+ms_excel_sheet_set_comment (MS_EXCEL_SHEET * sheet, int col, int row, char *text)
+{
+	if (text)
+	{
+		Cell *cell = sheet_cell_fetch (sheet->gnum_sheet, col, row);
+		sheet->blank = 0 ;
+		cell_set_comment (cell, text);
+	}
+}
+
+static void
+ms_excel_sheet_append_comment (MS_EXCEL_SHEET * sheet, int col, int row, char *text)
+{
+	if (text)
+	{
+		Cell *cell = sheet_cell_fetch (sheet->gnum_sheet, col, row);
+		if (cell->comment && cell->comment->comment &&
+		    cell->comment->comment->str) {
+			char *txt = g_strconcat (cell->comment->comment->str, txt, NULL);
+			sheet->blank = 0 ;
+			cell_set_comment (cell, txt);
+			g_free (txt);
+		}
+	}
+}
+
 static void
 ms_excel_sheet_set_index (MS_EXCEL_SHEET *ans, int idx)
 {
@@ -1206,7 +1273,9 @@ ms_excel_sheet_destroy (MS_EXCEL_SHEET * sheet)
 	g_hash_table_destroy (sheet->shared_formulae) ;
 	sheet->shared_formulae = NULL ;
 
-	sheet_destroy (sheet->gnum_sheet);
+	if (sheet->gnum_sheet)
+		sheet_destroy (sheet->gnum_sheet);
+	sheet->gnum_sheet = NULL ;
 	
 	g_free (sheet);
 }
@@ -1243,18 +1312,21 @@ ms_excel_workbook_new ()
 static void
 ms_excel_workbook_attach (MS_EXCEL_WORKBOOK * wb, MS_EXCEL_SHEET * ans)
 {
-	int    idx = 0 ;
-	GList *list = wb->excel_sheets ;
+	int    idx = 0;
+	GList *list = wb->excel_sheets;
+	g_return_if_fail (wb);
+	g_return_if_fail (ans);
 
 	workbook_attach_sheet (wb->gnum_wb, ans->gnum_sheet);
 	
 	while (list)
 	{
-		idx++ ;
-		list = list->next ;
+		g_return_if_fail (list->data != ans);
+		idx++;
+		list = list->next;
 	}
-	ms_excel_sheet_set_index (ans, idx) ;
-	wb->excel_sheets = g_list_append (wb->excel_sheets, ans) ;
+	ms_excel_sheet_set_index (ans, idx);
+	wb->excel_sheets = g_list_append (wb->excel_sheets, ans);
 }
 
 static void
@@ -1263,13 +1335,17 @@ ms_excel_workbook_detach (MS_EXCEL_WORKBOOK * wb, MS_EXCEL_SHEET * ans)
 	int    idx = 0 ;
 	GList *list = wb->excel_sheets ;
 
-	if (ans->gnum_sheet)
-		workbook_detach_sheet (wb->gnum_wb, ans->gnum_sheet);
+	if (ans->gnum_sheet) {
+		if (!workbook_detach_sheet (wb->gnum_wb, ans->gnum_sheet))
+			printf ("Error detaching sheet: '%s'\n",
+				ans->gnum_sheet->name?ans->gnum_sheet->name:"noname");
+	}
 	
 	while (list)
 		if (list->data == ans)
 		{
-			wb->excel_sheets = g_list_remove(wb->excel_sheets, list);
+			wb->excel_sheets = g_list_remove(wb->excel_sheets, list->data);
+			assert (g_list_find(wb->excel_sheets, list->data) == NULL) ;
 			return ;
 		}
 		else
@@ -1341,21 +1417,29 @@ biff_get_rk (guint8 *ptr)
 	LONG number;
 	LONG tmp[2];
 	double answer;
+	int li, hi;
 	enum eType {
-		eIEEE = 0, eIEEEx10 = 1, eInt = 2, eIntx100 = 3
+		eIEEE = 0, eIEEEx100 = 1, eInt = 2, eIntx100 = 3
 	} type;
 	
 	number = BIFF_GETLONG (ptr);
 	type = (number & 0x3);
+#if G_BYTE_ORDER != G_LITTLE_ENDIAN
+	li=1;
+	hi=0;
+#else
+	li=0; /* Intel */
+	hi=1;
+#endif
 	switch (type){
 	case eIEEE:
-		tmp[0] = 0;
-		tmp[1] = number & 0xfffffffc;
+		tmp[li] = 0;
+		tmp[hi] = *((LONG *)(ptr)) & 0xfffffffc;
 		answer = BIFF_GETDOUBLE (((BYTE *) tmp));
 		break;
-	case eIEEEx10:
-		tmp[0] = 0;
-		tmp[1] = number & 0xfffffffc;
+	case eIEEEx100:
+		tmp[li] = 0;
+		tmp[hi] = *((LONG *)(ptr)) & 0xfffffffc;
 		answer = BIFF_GETDOUBLE (((BYTE *) tmp));
 		answer /= 100.0;
 		break;
@@ -1465,20 +1549,22 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 		break ;
 	case BIFF_NUMBER:
 		{
-			char buf[65];
+			char buf[MS_EXCEL_DOUBLE_FORMAT_LEN];
 			double num = BIFF_GETDOUBLE (q->data + 6);	/*
 									 * FIXME GETDOUBLE is not endian independant 
 									 */
 /*			dump (q->data, q->length);
 			snprintf (buf, 64, "NUM %f", num); */
-			snprintf (buf, 64, "%f", num);
-			ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q), buf);
+			snprintf (buf, MS_EXCEL_DOUBLE_FORMAT_LEN-1,
+				  MS_EXCEL_DOUBLE_FORMAT, num);
+			ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q),
+					       EX_GETROW (q), buf);
 			break;
 		}
 	case BIFF_COLINFO: /* FIXME: See: S59D67.HTM */
 	{
-		int firstcol, lastcol, width, lp ;
-		guint16 cols_xf, options ;
+		int firstcol, lastcol, lp ;
+		guint16 cols_xf, options, width ;
 		int hidden, collapsed, outlining ;
 		firstcol = BIFF_GETWORD(q->data) ;
 		lastcol  = BIFF_GETWORD(q->data+2) ;
@@ -1492,9 +1578,10 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 
 		if (EXCEL_DEBUG>0 && BIFF_GETBYTE(q->data+10) != 0)
 			printf ("Odd Colinfo\n") ;
-		printf ("Column Formatting from col %d to %d of width %f characters\n",
-			firstcol, lastcol, width/256.0) ;
-		if ((int)width/256.0 == 0) {
+		if (EXCEL_DEBUG>0)
+			printf ("Column Formatting from col %d to %d of width %f characters\n",
+				firstcol, lastcol, width/256.0) ;
+		if (width>>8 == 0) {
 			printf ("FIXME: Hidden columns need implementing\n") ;
 			width=40.0 ;
 		}
@@ -1504,19 +1591,20 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 	}
 	case BIFF_RK: /* See: S59DDA.HTM */
 	{
-		char buf[65];
+		char buf[MS_EXCEL_DOUBLE_FORMAT_LEN];
 		
 /*		printf ("RK number : 0x%x, length 0x%x\n", q->opcode, q->length);
 		dump (q->data, q->length);*/
-		snprintf (buf, 64, "%f", biff_get_rk(q->data+6));
+		snprintf (buf, MS_EXCEL_DOUBLE_FORMAT_LEN-1,
+			  MS_EXCEL_DOUBLE_FORMAT, biff_get_rk(q->data+6));
 		ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q), buf);
 		break;
 	}
 	case BIFF_MULRK: /* S59DA8.HTM */
 	{
-		guint32 col, row, lastcol ;
-		char buf[65] ;
-		guint8 *ptr = q->data ;
+		guint32 col, row, lastcol;
+		char buf[MS_EXCEL_DOUBLE_FORMAT_LEN];
+		guint8 *ptr = q->data;
 
 /*		printf ("MULRK\n") ;
 		dump (q->data, q->length) ; */
@@ -1529,7 +1617,8 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 		g_assert (lastcol>=col) ;
 		while (col<=lastcol)
 		{ /* 2byte XF, 4 byte RK */
-			snprintf (buf, 64, "%f", biff_get_rk(ptr+2)) ;
+			snprintf (buf, MS_EXCEL_DOUBLE_FORMAT_LEN,
+				  MS_EXCEL_DOUBLE_FORMAT, biff_get_rk(ptr+2)) ;
 			ms_excel_sheet_insert(sheet, BIFF_GETWORD(ptr), col, row, buf) ;
 			col++ ;
 			ptr+= 6 ;
@@ -1579,19 +1668,19 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 					      data, data_len) ;
 		g_hash_table_insert (sheet->shared_formulae, &sf->key, sf) ;
 
-		printf ("Shared formula of extent %d %d %d %d\n",
-			array_col_first, array_row_first, array_col_last, array_row_last) ;
+		if (EXCEL_DEBUG>0)
+			printf ("Shared formula of extent %d %d %d %d\n",
+				array_col_first, array_row_first, array_col_last, array_row_last) ;
 		txt = ms_excel_parse_formula (sheet, data,
 					      array_col_first, array_row_first,
-					      array_col_first, array_row_first,
-					      data_len) ;
+					      1, data_len) ;
 		/* NB. This keeps the pre-set XF record */
 		cell = sheet_cell_fetch (sheet->gnum_sheet,
 					 array_col_first, array_row_first);
-		if (!cell)
-			break ;
-		cell_set_text_simple (cell, txt);
-		g_free(txt) ;
+		if (txt) {
+			cell_set_text_simple (cell, txt);
+			g_free(txt) ;
+		}
 		break ;
 	}
 	case BIFF_ARRAY: /* See: S59D57.HTM */
@@ -1618,7 +1707,7 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 			{
 				char *txt = ms_excel_parse_formula (sheet, data,
 								    xlp, ylp, 
-								    xlp, ylp, data_len) ;
+								    0, data_len) ;
 				/* NB. This keeps the pre-set XF record */
 				Cell *cell = sheet_cell_fetch (sheet->gnum_sheet, xlp, ylp);
 				if (cell)
@@ -1631,8 +1720,7 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 	{
 		char *txt = ms_excel_parse_formula (sheet, (q->data + 22),
 						    EX_GETCOL (q), EX_GETROW (q),
-						    EX_GETCOL (q), EX_GETROW (q),
-						    BIFF_GETWORD(q->data+20));
+						    0, BIFF_GETWORD(q->data+20));
 		ms_excel_sheet_insert (sheet, EX_GETXF(q), EX_GETCOL(q), EX_GETROW(q), txt) ;
 		g_free (txt) ;
 		break;
@@ -1752,8 +1840,8 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 		case BIFF_BOOLERR: /* S59D5F.HTM */
 		{
 			if (BIFF_GETBYTE(q->data + 7)) /* Error */
-				ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q)
-						       , EX_GETROW (q), 
+				ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q),
+						       EX_GETROW (q), 
 						       biff_get_error_text (BIFF_GETBYTE(q->data + 6))) ;
 			else /* Boolean */
 			{
@@ -1789,10 +1877,14 @@ ms_excel_read_sheet (MS_EXCEL_SHEET *sheet, BIFF_QUERY * q, MS_EXCEL_WORKBOOK *
 				printf ("Blank sheet\n");
 				ms_excel_workbook_detach (sheet->wb, sheet) ;
 				ms_excel_sheet_destroy (sheet) ;
+				sheet = NULL ;
 				return;
 			}
 			return;
 			break;
+		case BIFF_OBJ: /* See: ms-obj.c and S59DAD.HTM */
+			ms_obj_read_obj (sheet, q);
+			break;
 		case BIFF_SELECTION: /* S59DE2.HTM */
 		{
 			int pane_number ;
@@ -1825,6 +1917,40 @@ ms_excel_read_sheet (MS_EXCEL_SHEET *sheet, BIFF_QUERY * q, MS_EXCEL_WORKBOOK *
 			sheet_cursor_set (sheet->gnum_sheet, act_col, act_row, act_col, act_row, act_col, act_row) ;
 			break ;
 		}
+		case BIFF_MS_O_DRAWING: /* FIXME: See: ms-escher.c and S59DA4.HTM */
+			if (gnumeric_debugging>0)
+				ms_escher_hack_get_drawing (q);
+			break;
+		case BIFF_NOTE: /* See: S59DAB.HTM */
+		{
+			guint16 row = EX_GETROW(q);
+			guint16 col = EX_GETCOL(q);
+			if ( sheet->ver >= eBiffV8 ) {
+				guint16 options = BIFF_GETWORD(q->data+4);
+				guint16 obj_id  = BIFF_GETWORD(q->data+6);
+				guint16 author_len = BIFF_GETWORD(q->data+8);
+				char *author=biff_get_text(author_len%2?q->data+11:q->data+10,
+							   author_len, NULL);
+				int hidden;
+				if (options&0xffd)
+					printf ("FIXME: Error in options\n");
+				hidden = (options&0x2)==0;
+				if (EXCEL_DEBUG>0)
+					printf ("Comment at %d,%d id %d options 0x%x hidden %d by '%s'\n",
+						col, row, obj_id, options, hidden, author);
+			} else {
+				guint16 author_len = BIFF_GETWORD(q->data+4);
+				char *text=biff_get_text(q->data+6, author_len, NULL);
+				if (EXCEL_DEBUG>1)
+					printf ("Comment at %d,%d '%s'\n", col, row, text);
+
+				if (row==0xffff && col==0)
+					ms_excel_sheet_append_comment (sheet, col, row, text);
+				else
+					ms_excel_sheet_set_comment (sheet, col, row, text);
+			}
+			break;
+		}
 		default:
 			switch (q->opcode) {
 			case BIFF_WINDOW2: /* FIXME: see S59E18.HTM */
@@ -1851,6 +1977,7 @@ ms_excel_read_sheet (MS_EXCEL_SHEET *sheet, BIFF_QUERY * q, MS_EXCEL_WORKBOOK *
 	}
 	ms_excel_workbook_detach (sheet->wb, sheet) ;
 	ms_excel_sheet_destroy (sheet) ;
+	sheet = NULL ;
 	printf ("Error, hit end without EOF\n");
 	return;
 }
@@ -2011,8 +2138,10 @@ ms_excelReadWorkbook (MS_OLE * file)
 			case BIFF_PRECISION:	/*
 						 * FIXME: 
 						 */
-				printf ("Opcode : 0x%x, length 0x%x\n", q->opcode, q->length);
-				dump (q->data, q->length);
+				if (EXCEL_DEBUG>0) {
+					printf ("Opcode : 0x%x, length 0x%x\n", q->opcode, q->length);
+					dump (q->data, q->length);
+				}
 				break;
 			case BIFF_XF_OLD:	/*
 						 * FIXME: see S59E1E.HTM 
@@ -2022,7 +2151,7 @@ ms_excelReadWorkbook (MS_OLE * file)
 				break;
 			case BIFF_SST: /* see S59DE7.HTM */
 			{
-				guint32 length, k ;
+				guint32 length, k, tot_len ;
 				BYTE *tmp ;
 
 				if (EXCEL_DEBUG>4) {
@@ -2033,6 +2162,7 @@ ms_excelReadWorkbook (MS_OLE * file)
 				wb->global_strings = g_new (char *, wb->global_string_max) ;
 
 				tmp = q->data + 8 ;
+				tot_len = 8 ;
 				for (k = 0; k < wb->global_string_max; k++)
 				{
 					guint32 byte_len ;
@@ -2041,9 +2171,29 @@ ms_excelReadWorkbook (MS_OLE * file)
 					if (!wb->global_strings[k])
 						printf ("Blank string in table at : %d\n", k) ;
 					tmp += byte_len + 2 ;
+					tot_len += byte_len + 2 ;
+					if (tot_len > q->length) {
+						/*
+						  This means that somehow, the string table has been split
+						  Perhaps it is too big for a single biff record, or
+						  perhaps excel is just cussid. Either way a big pain.
+						*/
+						printf ("FIXME: Serious SST overrun lost %d of %d strings!\n",
+							wb->global_string_max - k, wb->global_string_max) ;
+						break ;
+					}
 				}
 				break;
 			}
+			case BIFF_EXTSST: /* See: S59D84 */
+				/* Can be safely ignored on read side */
+				break;
+			case BIFF_MS_O_DRAWING_GROUP: /* FIXME: See: S59DA5.HTM */
+				if (gnumeric_debugging>0) {
+					printf ("FIXME: MS Drawing Group\n");
+					ms_escher_hack_get_drawing (q);
+				}
+				break;
 			case BIFF_EXTERNSHEET:
 			{
 				if ( ver->version == eBiffV8 )
@@ -2095,7 +2245,8 @@ ms_excelReadWorkbook (MS_OLE * file)
 				g_hash_table_insert (wb->format_data, &d->idx, d) ;
 			}
 			case BIFF_EXTERNCOUNT: /* see S59D7D.HTM */
-				printf ("%d external references\n", BIFF_GETWORD(q->data)) ;
+				if (EXCEL_DEBUG>0)
+					printf ("%d external references\n", BIFF_GETWORD(q->data)) ;
 				break ;
 			default:
 				switch (q->opcode)
