@@ -159,7 +159,7 @@ FORMULA_FUNC_DATA formula_func_data[] =
 	{ "ATAN2", 2 },
 	{ "ASIN", 1 },
 	{ "ACOS", 1 },
-	{ "0x64", 8 },
+	{ "CHOOSE", -1 },
 	{ "HLOOKUP", -1 },
 	{ "VLOOKUP", -1 },
 	{ "0x67", 8 },
@@ -207,7 +207,7 @@ FORMULA_FUNC_DATA formula_func_data[] =
 	{ "0x91", 8 },
 	{ "0x92", 8 },
 	{ "0x93", 8 },
-	{ "0x94", 8 },
+	{ "INDIRECT", -1 },
 	{ "0x95", 8 },
 	{ "0x96", 8 },
 	{ "0x97", 8 },
@@ -427,19 +427,27 @@ FORMULA_FUNC_DATA formula_func_data[] =
  * storage structure.
  **/
 static CellRef *
-getRefV7(MS_EXCEL_SHEET *sheet, BYTE col, WORD gbitrw, int curcol, int currow)
+getRefV7(MS_EXCEL_SHEET *sheet, BYTE col, WORD gbitrw, int curcol, int currow, int shrfmla)
 {
 	CellRef *cr = (CellRef *)g_malloc(sizeof(CellRef)) ;
 	cr->col          = col ;
 	cr->row          = (gbitrw & 0x3fff) ;
 	cr->row_relative = (gbitrw & 0x8000)==0x8000 ;
 	cr->col_relative = (gbitrw & 0x4000)==0x4000 ;
+	cr->sheet = sheet->gnum_sheet ;
+	if (FORMULA_DEBUG>2)
+		printf ("7In : 0x%x, 0x%x  at %d, %d shared %d\n", col, gbitrw,
+			curcol, currow, shrfmla) ; 
+	if (shrfmla) {  /* FIXME: Brutal hack, docs are vague */
+		gint8 t = (cr->row&0x00ff);
+		cr->row = currow+t ;
+		t = (cr->col&0x00ff);
+		cr->col = curcol+t ;
+	}
 	if (cr->row_relative)
 		cr->row-= currow ;
 	if (cr->col_relative)
 		cr->col-= curcol ;
-	cr->sheet = sheet->gnum_sheet ;
-/*	printf ("7Out : %d, %d  at %d, %d\n", cr->col, cr->row, curcol, currow) ; */
 	return cr ;
 }
 /**
@@ -447,19 +455,34 @@ getRefV7(MS_EXCEL_SHEET *sheet, BYTE col, WORD gbitrw, int curcol, int currow)
  * storage structure.
  **/
 static CellRef *
-getRefV8(MS_EXCEL_SHEET *sheet, WORD row, WORD gbitcl, int curcol, int currow)
+getRefV8(MS_EXCEL_SHEET *sheet, WORD row, WORD gbitcl, int curcol, int currow, int shrfmla)
 {
 	CellRef *cr = (CellRef *)g_malloc(sizeof(CellRef)) ;
+	cr->sheet = sheet->gnum_sheet ;
+	if (FORMULA_DEBUG>2)
+		printf ("8In : 0x%x, 0x%x  at %d, %d shared %d\n", row, gbitcl,
+			curcol, currow, shrfmla) ;
 	cr->row          = row ;
 	cr->col          = (gbitcl & 0x3fff) ;
 	cr->row_relative = (gbitcl & 0x8000)==0x8000 ;
 	cr->col_relative = (gbitcl & 0x4000)==0x4000 ;
+	if (shrfmla) { /* FIXME: Brutal hack, docs are vague */
+		gint8 t = (cr->row&0x00ff);
+		if (FORMULA_DEBUG>2)
+			printf ("row off %d ",t);
+		cr->row = currow+t ;
+		t = (cr->col&0x00ff);
+		if (FORMULA_DEBUG>2)
+			printf ("col off %d\n",t);
+		cr->col = curcol+t ;
+	}
 	if (cr->row_relative)
 		cr->row-= currow ;
 	if (cr->col_relative)
 		cr->col-= curcol ;
-	cr->sheet = sheet->gnum_sheet ;
-/*	printf ("8Out : %d, %d  at %d, %d\n", cr->col, cr->row, curcol, currow) ; */
+	if (FORMULA_DEBUG>2)
+		printf ("Returns : %d,%d Rel:(%d %d)\n", cr->col, cr->row,
+			cr->col_relative, cr->row_relative);
 	return cr ;
 }
 
@@ -696,8 +719,7 @@ make_function (PARSE_LIST *stack, int fn_idx, int numargs)
  * Return a dynamicaly allocated string containing the formula, never NULL
  **/
 char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
-			      int fn_col, int fn_row, 
-			      int shr_col, int shr_row, guint16 length)
+			      int fn_col, int fn_row, int shared, guint16 length)
 {
 	Cell *cell ;
 	int len_left = length ;
@@ -734,21 +756,20 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 		switch (ptgbase)
 		{
 		case FORMULA_PTG_REFN:
-			printf ("REFN\n") ;
-			dump(mem, length) ;
-			break ;
 		case FORMULA_PTG_REF:
 		{
 			CellRef *ref ;
 			char *buffer ;
 			if (sheet->ver == eBiffV8)
 			{
-				ref = getRefV8 (sheet, BIFF_GETWORD(cur), BIFF_GETWORD(cur + 2), fn_col, fn_row) ;
+				ref = getRefV8 (sheet, BIFF_GETWORD(cur), BIFF_GETWORD(cur + 2),
+						fn_col, fn_row, shared) ;
 				ptg_length = 4 ;
 			}
 			else
 			{
-				ref = getRefV7 (sheet, BIFF_GETBYTE(cur+2), BIFF_GETWORD(cur), fn_col, fn_row) ;
+				ref = getRefV7 (sheet, BIFF_GETBYTE(cur+2), BIFF_GETWORD(cur),
+						fn_col, fn_row, shared) ;
 				ptg_length = 3 ;
 			}
 			buffer = cellref_name (ref, sheet->gnum_sheet, fn_col, fn_row) ;
@@ -793,7 +814,8 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 				char *ans, *intertxt, *ptr ;
 
 				intertxt = get_inter_sheet_ref (sheet->wb, extn_idx) ;
-				ref = getRefV8 (sheet, BIFF_GETWORD(cur+2), BIFF_GETWORD(cur + 4), fn_col, fn_row) ;
+				ref = getRefV8 (sheet, BIFF_GETWORD(cur+2), BIFF_GETWORD(cur + 4),
+						fn_col, fn_row, 0) ;
 				ptr = cellref_name (ref, sheet->gnum_sheet, fn_col, fn_row) ;
 
 				ans = g_new (char, strlen(intertxt)+strlen(ptr)+1) ;
@@ -829,8 +851,10 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 				CellRef *first, *last ;
 
 				intertxt = get_inter_sheet_ref (sheet->wb, extn_idx) ; 
-				first = getRefV8(sheet, BIFF_GETBYTE(cur+2), BIFF_GETWORD(cur+6), fn_col, fn_row) ;
-				last  = getRefV8(sheet, BIFF_GETBYTE(cur+4), BIFF_GETWORD(cur+8), fn_col, fn_row) ;
+				first = getRefV8(sheet, BIFF_GETBYTE(cur+2), BIFF_GETWORD(cur+6),
+						 fn_col, fn_row, 0) ;
+				last  = getRefV8(sheet, BIFF_GETBYTE(cur+4), BIFF_GETWORD(cur+8),
+						 fn_col, fn_row, 0) ;
 
 				fstr = cellref_name (first, sheet->gnum_sheet, fn_col, fn_row) ;
 				lstr = cellref_name (last, sheet->gnum_sheet, fn_col, fn_row) ;
@@ -862,23 +886,22 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 		}
 		break ;
 		case FORMULA_PTG_AREAN:
-			printf ("REFN\n") ;
-			dump(mem, length) ;			
-			break ;
 		case FORMULA_PTG_AREA:
 		{
 			CellRef *first, *last ;
 			char *fstr, *lstr, *buffer ;
 			if (sheet->ver == eBiffV8)
 			{
-				first = getRefV8(sheet, BIFF_GETBYTE(cur+0), BIFF_GETWORD(cur+4), fn_col, fn_row) ;
-				last  = getRefV8(sheet, BIFF_GETBYTE(cur+2), BIFF_GETWORD(cur+6), fn_col, fn_row) ;
+				first = getRefV8(sheet, BIFF_GETBYTE(cur+0), BIFF_GETWORD(cur+4),
+						 fn_col, fn_row, shared) ;
+				last  = getRefV8(sheet, BIFF_GETBYTE(cur+2), BIFF_GETWORD(cur+6),
+						 fn_col, fn_row, shared) ;
 				ptg_length = 8 ;
 			}
 			else
 			{
-				first = getRefV7(sheet, BIFF_GETBYTE(cur+4), BIFF_GETWORD(cur+0), fn_col, fn_row) ;
-				last  = getRefV7(sheet, BIFF_GETBYTE(cur+5), BIFF_GETWORD(cur+2), fn_col, fn_row) ;
+				first = getRefV7(sheet, BIFF_GETBYTE(cur+4), BIFF_GETWORD(cur+0), fn_col, fn_row, shared) ;
+				last  = getRefV7(sheet, BIFF_GETBYTE(cur+5), BIFF_GETWORD(cur+2), fn_col, fn_row, shared) ;
 				ptg_length = 6 ;
 			}
 			fstr = cellref_name (first, sheet->gnum_sheet, fn_col, fn_row) ;
@@ -894,8 +917,43 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 			g_free (lstr) ;
 			g_free (first) ;
 			g_free (last) ;
+			break ;
+		}
+		case FORMULA_PTG_ARRAY:
+		{
+			guint16 cols=BIFF_GETBYTE(cur+0);
+			guint16 rows=BIFF_GETWORD(cur+1);
+			guint16 lpx,lpy;
+			guint8 *data=cur+3;
+			GString *ans = g_string_new ("{");
+			ptg_length = 3;
+			if (cols==0) cols=256;
+			for (lpy=0;lpy<rows;lpy++) {
+				for (lpx=0;lpx<cols;lpx++) {
+					guint8 opts=BIFF_GETBYTE(data);
+					if (opts == 1) {
+						g_string_sprintfa (ans, MS_EXCEL_DOUBLE_FORMAT,
+								   BIFF_GETDOUBLE(data+1));
+						data+=9;
+						ptg_length+=9;
+					} else if (opts == 2) {
+						guint32 len;
+						g_string_append (ans, biff_get_text (data+2, BIFF_GETBYTE(data+1), &len));
+						g_string_append (ans, ",");
+						data+=len+2;
+						ptg_length+=2+len;
+					} else {
+						printf ("Duff type\n");
+						break;
+					}
+					g_string_append (ans, ";");
+				}
+			}
+			g_string_append (ans, "}");
+			parse_list_push_raw (stack, ans->str, NO_PRECEDENCE);
+			g_string_free (ans, FALSE);
+			break;
 		}
-		break ;
 		case FORMULA_PTG_FUNC:
 		{
 			if (!make_function (stack, BIFF_GETWORD(cur), -1)) error = 1 ;
@@ -916,13 +974,18 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 		case FORMULA_PTG_NAME:
 		{
 			guint16 name_idx ; /* 1 based */
+			char *txt;
 			if (sheet->ver == eBiffV8)
 				name_idx = BIFF_GETWORD(cur+2) ;
 			else
 				name_idx = BIFF_GETWORD(cur) ;
-			printf ("FIXME: Ptg Name: %d\n", name_idx) ;
-			dump(mem, length) ;
-			parse_list_push_raw (stack, g_strdup("Unknown name"), NO_PRECEDENCE) ;
+			txt = biff_name_data_get_name (sheet, name_idx);
+			if (!txt) {
+				printf ("FIXME: Ptg Name not found: %d\n", name_idx) ;
+				dump(mem, length) ;
+				parse_list_push_raw (stack, g_strdup("Unknown name"), NO_PRECEDENCE) ;
+			} else
+				parse_list_push_raw (stack, g_strdup(txt), NO_PRECEDENCE) ;
 		}
 		case FORMULA_PTG_EXP:
 		{
@@ -934,7 +997,7 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 							      fn_col, fn_row) ;
 			txt[0] = ' ' ; /* Kill '=' */
 			parse_list_push_raw (stack, txt, NO_PRECEDENCE) ;
-			ptg_length = 4 ;
+			ptg_length = length; /* Force it to be the only token 4 ; */
 			break ;
 		}
 		case FORMULA_PTG_PAREN:
@@ -962,13 +1025,40 @@ char *ms_excel_parse_formula (MS_EXCEL_SHEET *sheet, guint8 *mem,
 				}
 				if (w)
 					txt = ms_excel_parse_formula (sheet, cur+ptg_length,
-								      fn_col, fn_row, shr_col, shr_row,
+								      fn_col, fn_row, shared,
 								      w) ;
 				else
 					txt = g_strdup(" ") ;
+				g_assert (strlen(txt)>0) ;
 				txt[0] = ' ' ; /* Kill the = */
 				parse_list_push_raw (stack, txt, NO_PRECEDENCE) ;
 				ptg_length += w ;
+			} else if (grbit & 0x04) { /* AttrChoose 'optimised' my foot. */
+				guint16 len, lp;
+				guint32 offset=0;
+				guint8 *data=cur+3;
+				char *txt;
+
+				if (FORMULA_DEBUG>1) {
+					printf ("'Optimised' choose\n");
+					dump (mem,length);
+				}
+				for (lp=0;lp<w;lp++) { /* w = wCases */
+					offset= BIFF_GETWORD(data);
+					len = BIFF_GETWORD(data+2) - offset;
+					if (FORMULA_DEBUG>1)
+						printf ("Get from %d len %d [ = 0x%x ]\n",
+							ptg_length+offset, len, *(cur+ptg_length+offset));
+					txt = ms_excel_parse_formula (sheet, cur+ptg_length+offset,
+								      fn_col, fn_row, shared,
+								      len);
+					txt[0] = ' ';
+					data+=2;
+					if (FORMULA_DEBUG>1)
+						printf ("Parsed to '%s'\n", txt);
+					parse_list_push_raw (stack, txt, NO_PRECEDENCE);
+				}
+				ptg_length+=BIFF_GETWORD(data);
 			} else if (grbit & 0x08) { /* AttrGoto */
 				if (FORMULA_DEBUG>2) {
 					printf ("Goto %d: cur = 0x%x\n", w, (int)(cur-mem)) ;
