@@ -22,20 +22,20 @@
 #define OLE_DEBUG 0
 
 /* These take a _guint8_ pointer */
-#define GET_GUINT8(p)  (*(p+0))
-#define GET_GUINT16(p) (*(p+0)+(*(p+1)<<8))
-#define GET_GUINT32(p) (*(p+0)+ \
-		    (*(p+1)<<8)+ \
-		    (*(p+2)<<16)+ \
-		    (*(p+3)<<24))
+#define GET_GUINT8(p)  (*((const guint8 *)(p)+0))
+#define GET_GUINT16(p) (*((const guint8 *)(p)+0)+(*((const guint8 *)(p)+1)<<8))
+#define GET_GUINT32(p) (*((const guint8 *)(p)+0)+ \
+		    (*((const guint8 *)(p)+1)<<8)+ \
+		    (*((const guint8 *)(p)+2)<<16)+ \
+		    (*((const guint8 *)(p)+3)<<24))
 
-#define SET_GUINT8(p,n)  (*(p+0)=n)
-#define SET_GUINT16(p,n) ((*(p+0)=((n)&0xff)), \
-                          (*(p+1)=((n)>>8)&0xff))
-#define SET_GUINT32(p,n) ((*(p+0)=((n))&0xff), \
-                          (*(p+1)=((n)>>8)&0xff), \
-                          (*(p+2)=((n)>>16)&0xff), \
-                          (*(p+3)=((n)>>24)&0xff))
+#define SET_GUINT8(p,n)  (*((guint8 *)(p)+0)=n)
+#define SET_GUINT16(p,n) ((*((guint8 *)(p)+0)=((n)&0xff)), \
+                          (*((guint8 *)(p)+1)=((n)>>8)&0xff))
+#define SET_GUINT32(p,n) ((*((guint8 *)(p)+0)=((n))&0xff), \
+                          (*((guint8 *)(p)+1)=((n)>>8)&0xff), \
+                          (*((guint8 *)(p)+2)=((n)>>16)&0xff), \
+                          (*((guint8 *)(p)+3)=((n)>>24)&0xff))
 
 
 #define SPECIAL_BLOCK  0xfffffffd
@@ -159,6 +159,7 @@ pps_get_text (BYTE *ptr, int length)
 {
 	int lp, skip;
 	char *ans;
+	guint16 c;
 	BYTE *inb;
 	
 	if (!length) 
@@ -166,14 +167,14 @@ pps_get_text (BYTE *ptr, int length)
 	
 	ans = (char *)g_malloc (sizeof(char) * length + 1);
 	
-	skip = (ptr[0] < 0x30); /* Magic unicode number */
-	if (skip)
+	c = GET_GUINT16(ptr);
+	if (c<0x30) /* Magic unicode number I made up */
 		inb = ptr + 2;
 	else
 		inb = ptr;
-	for (lp=0;lp<length;lp++)
-	{
-		ans[lp] = (char) *inb;
+	for (lp=0;lp<length;lp++) {
+		c = GET_GUINT16(inb);
+		ans[lp] = (char)c;
 		inb+=2;
 	}
 	ans[lp] = 0;
@@ -529,7 +530,7 @@ ms_ole_new (const char *name)
 		ms_ole_destroy(f);
 		return 0;
 	}
-	printf ("New OLE file\n");
+	printf ("New OLE file '%s'\n", name);
 	return f;
 }
 
@@ -1228,6 +1229,16 @@ ms_ole_stream_open (MS_OLE_DIRECTORY *d, char mode)
 	return s;
 }
 
+/* FIXME: This needs to be more cunning and have new write / read
+   functions that inser CONTINUE records etc. */
+static MS_OLE_STREAM *
+ms_ole_stream_duplicate (MS_OLE_STREAM *s)
+{
+	MS_OLE_STREAM *ans = g_new (MS_OLE_STREAM, 1);
+	memcpy (ans, s, sizeof(MS_OLE_STREAM));
+	return ans;
+}
+
 void
 ms_ole_stream_close (MS_OLE_STREAM *s)
 {
@@ -1255,7 +1266,8 @@ ms_ole_directory_new (MS_OLE *f)
 static void
 directory_setup (MS_OLE_DIRECTORY *d)
 {
-	printf ("Setup pps = %d\n", d->pps);
+	if (OLE_DEBUG>0)
+		printf ("Setup pps = %d\n", d->pps);
 	g_free (d->name);
 	d->name   = PPS_NAME(d->file, d->pps);
 	d->type   = PPS_GET_TYPE(d->file, d->pps);
@@ -1290,7 +1302,8 @@ ms_ole_directory_next (MS_OLE_DIRECTORY *d)
 		    tmp != PPS_END_OF_CHAIN)
 		{
 			offset--;
-			printf ("Back trace by %d\n", offset);
+			if (OLE_DEBUG>0)
+				printf ("Back trace by %d\n", offset);
 			tmp = d->primary_entry;
 			while (offset > 0)
 			{
@@ -1308,11 +1321,13 @@ ms_ole_directory_next (MS_OLE_DIRECTORY *d)
 	if (tmp == PPS_END_OF_CHAIN)
 		return 0;
 
-	printf ("Forward trace\n");
+	if (OLE_DEBUG>0)
+		printf ("Forward trace\n");
 	d->pps = tmp;
 
 	directory_setup(d);
-	printf ("Next '%s' %d %d\n", d->name, d->type, d->length);
+	if (OLE_DEBUG>0)
+		printf ("Next '%s' %d %d\n", d->name, d->type, d->length);
 	return 1;
 }
 
@@ -1490,6 +1505,7 @@ ms_biff_query_copy (const BIFF_QUERY *p)
 		bf->data = (guint8 *)g_malloc (p->length);
 		memcpy (bf->data, p->data, p->length);
 	}
+	bf->pos=ms_ole_stream_duplicate (p->pos);
 	return bf;
 }
 
@@ -1545,6 +1561,21 @@ ms_biff_query_destroy (BIFF_QUERY *bq)
 	}
 }
 
+/* FIXME: Too nasty ! */
+MS_OLE_STREAM *
+ms_biff_query_data_to_stream (BIFF_QUERY *bq)
+{
+	MS_OLE_STREAM *ans=ms_ole_stream_duplicate (bq->pos);
+/*	ans->advance(ans, -bq->length); 
+	This will never work !
+*/
+	/* Hack size down to biff length */
+	/* Can't be done non-destructively ! sod ! */
+	/* Should cut the length down a lot, hope we can know where
+	   the end is somehow */
+	return ans;
+}
+
 #if G_BYTE_ORDER != G_LITTLE_ENDIAN
 double biff_getdouble(guint8 *p)
 {
