@@ -601,8 +601,13 @@ cell_set_text_simple (Cell *cell, char *text)
 				/* falldown */
 				
 			case 'E': case 'e': case '+': case ':': case '.': case ',':
-				if (*p == 'e' || *p == 'E')
+				if (*p == 'e' || *p == 'E') {
 					seen_exp = TRUE;
+
+					/* Lookahead */
+					if (*(p+1) == '-')
+						p++; 
+				}
 				
 				if (*p == ',' || *p == '.')
 					if (*lconv->decimal_point != *p){
@@ -791,7 +796,9 @@ cell_destroy (Cell *cell)
 		string_unref  (cell->text);
 
 	style_destroy (cell->style);
-	value_release (cell->value);
+
+	if (cell->value)
+		value_release (cell->value);
 
 	g_free (cell);
 }
