@@ -182,6 +182,10 @@ new_node (GList *list, char *data, char *p, int col, int row)
 	CellCopy *c_copy;
 	char *text;
 
+	/* Eliminate spaces */
+	while (*data == ' ' && *data)
+		data++;
+	
 	text = g_malloc (p-data+1);
 	text = strncpy (text, data, p-data);
 	text [p-data] = 0;
@@ -223,9 +227,9 @@ x_selection_to_cell_region (char *data, int len)
 				rows++;
 				cur_col = 0;
 			} else {
+				cur_col++;
 				if (cur_col > cols)
 					cols = cur_col;
-				cur_col++;
 			}
 
 			data = p+1;
@@ -233,9 +237,13 @@ x_selection_to_cell_region (char *data, int len)
 	}
 
 	/* Handle the remainings */
-	if (p != data)
+	if (p != data){
 		list = new_node (list, data, p, cur_col, rows);
-
+		cur_col++;
+		if (cur_col > cols)
+			cols = cur_col;
+	}
+	
 	/* Return the CellRegion */
 	cr = g_new (CellRegion, 1);	
 	cr->list = list;
