@@ -771,7 +771,7 @@ gnumeric_correl (void *tsheet, GList *expr_node_list, int eval_col, int eval_row
 {
 	stat_correl_t pr;
 	Sheet *sheet = (Sheet *) tsheet;
-	float_t sum;
+	float_t sum, tmp;
 	int     count;
 	GSList  *list1, *list2;
 
@@ -813,9 +813,13 @@ gnumeric_correl (void *tsheet, GList *expr_node_list, int eval_col, int eval_row
 	g_slist_free (pr.array1);
 	g_slist_free (pr.array2);
 
-	return value_float ((sum - (pr.sum1*pr.sum2/pr.count)) /
-			    sqrt((pr.sqrsum1-(pr.sum1*pr.sum1)/pr.count) *
-				 (pr.sqrsum2-(pr.sum2*pr.sum2)/pr.count)));
+	tmp = (pr.sqrsum1-(pr.sum1*pr.sum1)/pr.count) *
+	        (pr.sqrsum2-(pr.sum2*pr.sum2)/pr.count);
+	if (tmp == 0)
+	        return value_float (0);
+	else
+	        return value_float ((sum - (pr.sum1*pr.sum2/pr.count)) /
+				    sqrt(tmp));
 }
 
 static char *help_negbinomdist = {
@@ -883,6 +887,58 @@ gnumeric_normsdist (struct FunctionDefinition *i, Value *argv [], char **error_s
 	return value_float (phi (x));
 }
 
+static char *help_normsinv = {
+       N_("@FUNCTION=NORMSINV\n"
+          "@SYNTAX=NORMSINV(p)\n"
+
+          "@DESCRIPTION="
+          "The NORMSINV function returns the inverse of the standard normal "
+	  "cumulative distribution. @p is the given probability corresponding "
+	  "to the normal distribution. NORMSINV uses an iterative algorithm "
+	  "for calculating the result. If NORMSINV does not converge "
+	  "(accuracy within +/- 3x10^7) after 100 iterations, the function "
+	  "returns #N/A! error. "
+          "\n"
+	  "If @p < 0 or @p > 1 NORMSINV returns #NUM! error. "
+	  "\n"
+          "@SEEALSO=NORMDIST,NORMINV,NORMSDIST,STANDARDIZE,ZTEST")
+};
+
+static Value *
+gnumeric_normsinv (struct FunctionDefinition *i, Value *argv [], char **error_string)
+{
+        const int left = 1;
+	const int right = 2;
+        const int iterations = 100;
+	const float_t accuracy_limit = 0.00000003;
+        float_t p, p_test, x = 0, step = 1;
+	int     n, dir = 0;
+
+        p = value_get_as_double (argv [0]);
+	if (p < 0 || p > 1) {
+		*error_string = _("#NUM!");
+		return NULL;
+	}
+	for (n=0; n<iterations; n++) {
+	        p_test = phi(x);
+		if (fabs(p - p_test) < accuracy_limit)
+		        return value_float (x);
+		if (p < p_test) {
+		        if (dir == right)
+			        step /= 2;
+		        x -= step;
+			dir = left;
+		} else {
+		        if (dir == left)
+			        step /= 2;
+		        x += step;
+			dir = right;
+		}
+	}
+	*error_string = _("#N/A!");
+	return NULL;
+}
+
 static char *help_lognormdist = {
        N_("@FUNCTION=LOGNORMDIST\n"
           "@SYNTAX=LOGNORMDIST(x,mean,stdev)\n"
@@ -922,6 +978,65 @@ gnumeric_lognormdist (struct FunctionDefinition *i, Value *argv [], char **error
 	return value_float (phi (x));
 }
 
+static char *help_loginv = {
+       N_("@FUNCTION=LOGINV\n"
+          "@SYNTAX=LOGINV(p,mean,stdev)\n"
+
+          "@DESCRIPTION="
+          "The LOGINV function returns the inverse of the lognormal "
+	  "cumulative distribution. @p is the given probability corresponding "
+	  "to the normal distribution, @mean is the arithmetic mean of the "
+	  "distribution, and @stdev is the standard deviation of the "
+	  "distribution. LOGINV uses an iterative algorithm "
+	  "for calculating the result. If LOGINV does not converge "
+	  "(accuracy within +/- 3x10^7) after 100 iterations, the function "
+	  "returns #N/A! error. "
+          "\n"
+	  "If @p < 0 or @p > 1 or @stdev <= 0 LOGINV returns #NUM! error. "
+	  "\n"
+          "@SEEALSO=EXP,LN,LOG,LOG10,LOGNORMDIST")
+};
+
+static Value *
+gnumeric_loginv (struct FunctionDefinition *i, Value *argv [], char **error_string)
+{
+        const int left = 1;
+	const int right = 2;
+        const int iterations = 100;
+	const float_t accuracy_limit = 0.00000003;
+        float_t p, p_test, x, step;
+	float_t mean, stdev;
+	int     n, dir = 0;
+
+        p = value_get_as_double (argv [0]);
+        mean = value_get_as_double (argv [1]);
+        stdev = value_get_as_double (argv [2]);
+	if (p < 0 || p > 1 || stdev <= 0) {
+		*error_string = _("#NUM!");
+		return NULL;
+	}
+	x = 0;
+	step = 1;
+	for (n=0; n<iterations; n++) {
+	        p_test = phi(x);
+		if (fabs(p - p_test) < accuracy_limit)
+		        return value_float (exp(mean+stdev*x));
+		if (p < p_test) {
+		        if (dir == right)
+			        step /= 2;
+		        x -= step;
+			dir = left;
+		} else {
+		        if (dir == left)
+			        step /= 2;
+		        x += step;
+			dir = right;
+		}
+	}
+	*error_string = _("#N/A!");
+	return NULL;
+}
+
 static char *help_fisherinv = {
        N_("@FUNCTION=FISHERINV\n"
           "@SYNTAX=FISHERINV(y)\n"
@@ -1621,6 +1736,65 @@ gnumeric_normdist (struct FunctionDefinition *i, Value *argv [], char **error_st
         }
 }
 
+static char *help_norminv = {
+       N_("@FUNCTION=NORMINV\n"
+          "@SYNTAX=NORMINV(p,mean,stdev)\n"
+
+          "@DESCRIPTION="
+          "The NORMINV function returns the inverse of the normal "
+	  "cumulative distribution. @p is the given probability corresponding "
+	  "to the normal distribution, @mean is the arithmetic mean of the "
+	  "distribution, and @stdev is the standard deviation of the "
+	  "distribution. NORMINV uses an iterative algorithm "
+	  "for calculating the result. If NORMINV does not converge "
+	  "(accuracy within +/- 3x10^7) after 100 iterations, the function "
+	  "returns #N/A! error. "
+          "\n"
+	  "If @p < 0 or @p > 1 or @stdev <= 0 NORMINV returns #NUM! error. "
+	  "\n"
+          "@SEEALSO=NORMDIST,NORMSDIST,NORMSINV,STANDARDIZE,ZTEST")
+};
+
+static Value *
+gnumeric_norminv (struct FunctionDefinition *i, Value *argv [], char **error_string)
+{
+        const int left = 1;
+	const int right = 2;
+        const int iterations = 100;
+	const float_t accuracy_limit = 0.00000003;
+        float_t p, p_test, x, step;
+	float_t mean, stdev;
+	int     n, dir = 0;
+
+        p = value_get_as_double (argv [0]);
+	mean = value_get_as_double (argv [1]);
+	stdev = value_get_as_double (argv [2]);
+	if (p < 0 || p > 1 || stdev <= 0) {
+		*error_string = _("#NUM!");
+		return NULL;
+	}
+	x = mean;
+	step = stdev;
+	for (n=0; n<iterations; n++) {
+	        p_test = phi((x-mean) / stdev);
+		if (fabs(p - p_test) < accuracy_limit)
+		        return value_float (x);
+		if (p < p_test) {
+		        if (dir == right)
+			        step /= 2;
+		        x -= step;
+			dir = left;
+		} else {
+		        if (dir == left)
+			        step /= 2;
+		        x += step;
+			dir = right;
+		}
+	}
+	*error_string = _("#N/A!");
+	return NULL;
+}
+
 static char *help_kurt = {
         N_("@FUNCTION=KURT\n"
            "@SYNTAX=KURT(n1, n2, ...)\n"
@@ -1976,6 +2150,72 @@ gnumeric_pearson (void *tsheet, GList *expr_node_list, int eval_col, int eval_ro
 				 (pr.count*pr.sqrsum2 - pr.sum2*pr.sum2)));
 }
 
+static char *help_rsq = {
+	N_("@FUNCTION=RSQ\n"
+	   "@SYNTAX=RSQ(array1,array2)\n"
+
+	   "@DESCRIPTION="
+	   "RSQ returns the square of the Pearson correllation coefficient "
+	   "of two data sets. "
+	   "\n"
+	   "Strings and empty cells are simply ignored."
+	   "\n"
+	   "@SEEALSO=CORREL,COVAR,INTERPCEPT,LINEST,LOGEST,PEARSON,SLOPE,"
+	   "STEYX,TREND")
+};
+
+static Value *
+gnumeric_rsq (void *tsheet, GList *expr_node_list, int eval_col, int eval_row, char **error_string)
+{
+	stat_correl_t pr;
+	Sheet *sheet = (Sheet *) tsheet;
+	float_t sum, r;
+	int     count;
+	GSList  *list1, *list2;
+
+	pr.first   = TRUE;
+	count = value_get_as_int(gnumeric_count
+	        (tsheet, expr_node_list, eval_col, eval_row, error_string));
+	if (count % 2 > 0){
+		*error_string = _("#NUM!");
+		return NULL;
+	}
+	pr.count   = count / 2;
+	pr.num     = 0;
+	pr.sum1    = 0.0;
+	pr.sum2    = 0.0;
+	pr.sqrsum1 = 0.0;
+	pr.sqrsum2 = 0.0;
+	pr.array1  = NULL;
+	pr.array2  = NULL;
+
+	function_iterate_argument_values (sheet, callback_function_correl,
+					  &pr, expr_node_list,
+					  eval_col, eval_row, error_string);
+	list1 = pr.array1;
+	list2 = pr.array2;
+	sum = 0.0;
+
+	while (list1 != NULL && list2 != NULL){
+	        gpointer x, y;
+		x = list1->data;
+		y = list2->data;
+	        sum += (*((float_t *) x)) * (*((float_t *) y));
+		g_free(x);
+		g_free(y);
+		list1 = list1->next;
+		list2 = list2->next;
+	}
+
+	g_slist_free(pr.array1);
+	g_slist_free(pr.array2);
+
+	r = (((pr.count*sum - pr.sum1*pr.sum2)) /
+	     sqrt((pr.count*pr.sqrsum1 - pr.sum1*pr.sum1) *
+		  (pr.count*pr.sqrsum2 - pr.sum2*pr.sum2)));
+	return value_float (r * r);
+}
+
 static char *help_median = {
        N_("@FUNCTION=MEDIAN\n"
           "@SYNTAX=MEDIAN(n1, n2, ...)\n"
@@ -2198,6 +2438,93 @@ gnumeric_small (void *tsheet, GList *expr_node_list, int eval_col, int eval_row,
 	return value_float (r);
 }
 
+static char *help_ztest = {
+	N_("@FUNCTION=ZTEST\n"
+	   "@SYNTAX=ZTEST(ref,x)\n"
+
+	   "@DESCRIPTION="
+	   "ZTEST returns the two-tailed probability of a z-test. "
+	   "\n"
+	   "@ref is the data set and @x is the value to be tested. "
+	   "\n"
+	   "If ref contains less than two data items ZTEST "
+	   "returns #DIV/0! error. "
+	   "\n"
+	   "@SEEALSO=CONFIDENCE,NORMDIST,NORMINV,NORMSDIST,NORMSINV,"
+	   "STANDARDIZE")
+};
+
+typedef struct {
+	guint32 num;
+        float_t x;
+        float_t sum;
+        float_t sqrsum;
+} stat_ztest_t;
+
+static int
+callback_function_ztest (Sheet *sheet, Value *value, char **error_string, void *closure)
+{
+	stat_ztest_t *mm = closure;
+	float_t last;
+
+	switch (value->type){
+	case VALUE_INTEGER:
+	        last = value->v.v_int;
+		break;
+	case VALUE_FLOAT:
+	        last = value->v.v_float;
+		break;
+	default:
+		return FALSE;
+	}
+	if (mm->num == 0)
+	        mm->x = last;
+	else {
+	        mm->sum += mm->x;
+		mm->sqrsum += mm->x * mm->x;
+		mm->x = last;
+	}
+	mm->num++;
+	return TRUE;
+}
+
+static Value *
+gnumeric_ztest (void *tsheet, GList *expr_node_list, int eval_col, int eval_row, char **error_string)
+{
+	stat_ztest_t p;
+	Sheet        *sheet = (Sheet *) tsheet;
+	int          status;
+	float_t      stdev;
+
+	p.num    = 0;
+	p.sum    = 0;
+	p.sqrsum = 0;
+
+	status = function_iterate_argument_values (sheet,
+						   callback_function_ztest,
+						   &p, expr_node_list,
+						   eval_col, eval_row,
+						   error_string);
+
+	if (status == FALSE) {
+	        *error_string = _("#VALUE!");
+		return NULL;
+	}
+	p.num--;
+	if (p.num < 2) {
+	        *error_string = _("#DIV/0!");
+		return NULL;
+	}
+	stdev = sqrt((p.sqrsum - p.sum*p.sum/p.num) / (p.num - 1));
+	if (stdev == 0) {
+	        *error_string = _("#DIV/0!");
+		return NULL;
+	}
+
+	return value_float (1 - phi ((p.sum/p.num - p.x) /
+				     (stdev / sqrt(p.num))));
+}
+
 
 FunctionDefinition stat_functions [] = {
         { "avedev",    0,      "",          &help_avedev,    gnumeric_avedev, NULL },
@@ -2218,14 +2545,18 @@ FunctionDefinition stat_functions [] = {
 	{ "hypgeomdist", "ffff", "x,n,M,N", &help_hypgeomdist, NULL, gnumeric_hypgeomdist },
         { "kurt",      0,      "",          &help_kurt,      gnumeric_kurt, NULL },
 	{ "large",  0,      "",             &help_large,  gnumeric_large, NULL },
+	{ "loginv",  "fff",  "",            &help_loginv, NULL, gnumeric_loginv },
 	{ "lognormdist",  "fff",  "",       &help_lognormdist, NULL, gnumeric_lognormdist },
 	{ "median",    0,      "",          &help_median,    gnumeric_median, NULL },
 	{ "mode",      0,      "",          &help_mode,   gnumeric_mode, NULL },
 	{ "negbinomdist", "fff", "f,t,p",   &help_negbinomdist, NULL, gnumeric_negbinomdist },
 	{ "normdist",   "fffb",  "",        &help_normdist,  NULL, gnumeric_normdist },
+	{ "norminv",    "fff",  "",         &help_norminv,  NULL, gnumeric_norminv },
 	{ "normsdist",  "f",  "",           &help_normsdist,  NULL, gnumeric_normsdist },
+	{ "normsinv",  "f",  "",            &help_normsinv,  NULL, gnumeric_normsinv },
 	{ "pearson",   0,      "",          &help_pearson,   gnumeric_pearson, NULL },
 	{ "rank",      0,      "",          &help_rank,      gnumeric_rank, NULL },
+	{ "rsq",       0,      "",          &help_rsq,      gnumeric_rsq, NULL },
 	{ "skew",      0,      "",          &help_skew,      gnumeric_skew, NULL },
 	{ "small",  0,      "",             &help_small,  gnumeric_small, NULL },
 	{ "standardize", "fff",  "x,mean,stddev", &help_standardize, NULL, gnumeric_standardize },
@@ -2235,5 +2566,6 @@ FunctionDefinition stat_functions [] = {
 	{ "var",       0,      "",          &help_var,       gnumeric_var, NULL },
 	{ "varp",      0,      "",          &help_varp,      gnumeric_varp, NULL },
         { "weibull", "fffb",  "",           &help_weibull, NULL, gnumeric_weibull },
+	{ "ztest",  0,      "",             &help_ztest,  gnumeric_ztest, NULL },
 	{ NULL, NULL },
 };
