@@ -14,7 +14,7 @@
 #include "func.h"
 
 /* The list of categories */
-static GPtrArray *categories = NULL ;
+static GPtrArray *categories = NULL;
 
 typedef struct {
 	FunctionIterateCallback  callback;
@@ -73,19 +73,21 @@ function_iterate_do_value (Sheet                   *sheet,
 			break;
 			
 	case VALUE_ARRAY:
-		for (list = value->v.array; list; list = list->next){
-			Value *array_v = (Value *) list->data;
+	{
+		int x, y;
 
-			ret = function_iterate_do_value (
-				sheet, callback, closure,
-				eval_col, eval_row,
-				array_v, error_string);
-			
-			if (ret == FALSE)
-				return FALSE;
+		for (x = 0; x < value->v.array.x; x++){
+			for (y = 0; y < value->v.array.y; y++){
+				ret = function_iterate_do_value (
+					sheet, callback, closure,
+					eval_col, eval_row,
+					&value->v.array.vals [x][y], error_string);
+				if (ret == FALSE)
+					return FALSE;
+			}
 		}
 		break;
-
+	}
 	case VALUE_CELLRANGE: {
 		IterateCallbackClosure data;
 		int start_col, start_row, end_col, end_row;
@@ -143,7 +145,7 @@ function_iterate_argument_values (Sheet                   *sheet,
 }
 
 GPtrArray *
-function_categories_get ()
+function_categories_get (void)
 {
 	return categories;
 }
@@ -153,7 +155,7 @@ tokenized_help_new (FunctionDefinition *fd)
 {
 	TokenizedHelp *tok;
 
-	g_return_if_fail (fd != NULL);
+	g_return_val_if_fail (fd != NULL, NULL);
 
 	tok = g_new (TokenizedHelp, 1);
 
@@ -190,7 +192,7 @@ tokenized_help_new (FunctionDefinition *fd)
 		tok->sections = NULL;
 	}
 	
-	return tok ;
+	return tok;
 }
 
 /**
