@@ -69,7 +69,7 @@ typedef struct {
 	FunctionDefinition *functions;
 } FunctionCategory;
 
-extern GPtrArray *function_categories_get ();
+GPtrArray *function_categories_get (void);
 
 typedef struct {
 	GPtrArray *sections ;
@@ -92,5 +92,7 @@ Value *gnumeric_count       (void *tsheet, GList *expr_node_list,
 			     int eval_col, int eval_row,
 			     char **error_string);
 
+int solver_simplex(Workbook *wb, Sheet *sheet);
+
 #endif /* GNUMERIC_FUNC_H */
 
