@@ -2,7 +2,7 @@
 #define GNUMERIC_ITEM_BAR_H
 
 #define ITEM_BAR(obj)          (GTK_CHECK_CAST((obj), item_bar_get_type (), ItemBar))
-#define ITEM_BAR_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_bar_get_type ()))
+#define ITEM_BAR_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_bar_get_type (), ItemBarType))
 #define IS_ITEM_BAR(o)         (GTK_CHECK_TYPE((o), item_bar_get_type ()))
 
 #define ITEM_BAR_RESIZING(x) (ITEM_BAR(x)->resize_pos != -1)
