@@ -2,7 +2,7 @@
 #define GNUMERIC_ITEM_GRID_H
 
 #define ITEM_GRID(obj)          (GTK_CHECK_CAST((obj), item_grid_get_type (), ItemGrid))
-#define ITEM_GRID_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_grid_get_type ()))
+#define ITEM_GRID_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_grid_get_type (), ItemGridClass))
 #define IS_ITEM_GRID(o)         (GTK_CHECK_TYPE((o), item_grid_get_type ()))
 
 typedef enum {
