@@ -15,6 +15,9 @@
 #include "eval.h"
 #include "number-match.h"
 #include "format.h"
+#ifdef ENABLE_BONOBO
+#    include <libgnorba/gnorba.h>
+#endif
 
 #define GNUMERIC_SHEET_VIEW(p) GNUMERIC_SHEET (SHEET_VIEW(p)->sheet_view);
 
@@ -138,7 +141,7 @@ sheet_rename (Sheet *sheet, const char *new_name)
 }
 
 Sheet *
-sheet_new (Workbook *wb, char *name)
+sheet_new (Workbook *wb, const char *name)
 {
 	GtkWidget *sheet_view;
 	Sheet *sheet;
@@ -253,28 +256,15 @@ sheet_set_zoom_factor (Sheet *sheet, double factor)
 	}
 }
 
-/*
- * Duplicates a column or row
- */
-ColRowInfo *
-sheet_duplicate_colrow (ColRowInfo *original)
-{
-	ColRowInfo *info = g_new (ColRowInfo, 1);
-
-	*info = *original;
-	
-	return info;
-}
-
 ColRowInfo *
 sheet_row_new (Sheet *sheet)
 {
-	ColRowInfo *ri;
+	ColRowInfo *ri = g_new (ColRowInfo, 1);
 
 	g_return_val_if_fail (sheet != NULL, NULL);
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
-	ri = sheet_duplicate_colrow (&sheet->default_row_style);
+	*ri = sheet->default_row_style;
 	row_init_span (ri);
 
 	return ri;
@@ -283,12 +273,12 @@ sheet_row_new (Sheet *sheet)
 ColRowInfo *
 sheet_col_new (Sheet *sheet)
 {
-	ColRowInfo *ci;
+	ColRowInfo *ci = g_new (ColRowInfo, 1);
 	
 	g_return_val_if_fail (sheet != NULL, NULL);
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 	
-	ci = sheet_duplicate_colrow (&sheet->default_col_style);
+	*ci = sheet->default_col_style;
 	ci->data = NULL;
 	
 	return ci;
@@ -749,7 +739,7 @@ sheet_set_text (Sheet *sheet, int col, int row, char *str)
 		/* Compare it */
 		if (strcasecmp (new_text, text) == 0){
 			cell_set_format_simple (cell, format);
-			sprintf (buffer, "%g", v);
+			sprintf (buffer, "%f", v);
 			cell_set_text (cell, buffer);
 			text_set = TRUE;
 		}
@@ -1029,7 +1019,7 @@ sheet_selection_first_range (Sheet *sheet,
 	if (!sheet->selections)
 		return 0;
 
-	l = g_list_first (sheet->selections) ;
+	l = g_list_first (sheet->selections);
 	if (!l || !l->data)
 		return 0;
 
@@ -1918,7 +1908,7 @@ fail_if_not_selected (Sheet *sheet, int col, int row, Cell *cell, void *user_dat
  	if (!sheet_selection_is_cell_selected (sheet, col, row))
   		return FALSE;
  	else
- 		return TRUE ;
+ 		return TRUE;
 }
 
 /**
@@ -2047,7 +2037,7 @@ sheet_cell_remove_from_hash (Sheet *sheet, Cell *cell)
 		g_free (original_key);
 	}
 	else
-		g_warning ("Cell not in hash table |\n") ;
+		g_warning ("Cell not in hash table |\n");
 }
 
 static void
@@ -2157,6 +2147,10 @@ sheet_cell_formula_unlink (Cell *cell)
 	sheet = cell->sheet;
 	cell_drop_dependencies (cell);
 	sheet->workbook->formula_cell_list = g_list_remove (sheet->workbook->formula_cell_list, cell);
+
+	/* Just an optimization to avoid an expensive list lookup */
+	if (cell->flags & CELL_QUEUED_FOR_RECALC)
+		cell_unqueue_from_recalc (cell);
 }
 
 /**
@@ -2168,16 +2162,18 @@ static void
 sheet_col_destroy (Sheet *sheet, ColRowInfo *ci)
 {
 	GList *l;
+	g_return_if_fail (ci);
 
 	for (l = ci->data; l; l = l->next){
 		Cell *cell = l->data;
-
+		l->data = NULL;
 		sheet_cell_remove_internal (sheet, cell);
 		cell_destroy (cell);
 	}
 	
 	sheet->cols_info = g_list_remove (sheet->cols_info, ci);
 	g_list_free (ci->data);
+	ci->data = NULL;
 	g_free (ci);
 }
 
@@ -2210,13 +2206,13 @@ sheet_destroy_styles (Sheet *sheet)
 static void
 sheet_destroy_columns_and_rows (Sheet *sheet)
 {
-	GList *l;
+	while (sheet->cols_info)
+		sheet_col_destroy (sheet, sheet->cols_info->data);
+	sheet->cols_info = NULL;
 
-	for (l = sheet->cols_info; l; l = l->next)
-		sheet_col_destroy (sheet, l->data);
-
-	for (l = sheet->rows_info; l; l = l->next)
-		sheet_row_destroy (sheet, l->data);
+	while (sheet->rows_info)
+		sheet_row_destroy (sheet, sheet->rows_info->data);
+	sheet->rows_info = NULL;
 }
 
 /**
@@ -2236,7 +2232,8 @@ sheet_destroy (Sheet *sheet)
 	g_assert (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet)); 
 	g_return_if_fail (sheet->workbook == NULL);
-			  
+
+	g_warning ("Reminder: need to destroy SheetObjects");
 	sheet_selections_free (sheet);
 	g_free (sheet->name);
 	
@@ -2246,14 +2243,16 @@ sheet_destroy (Sheet *sheet)
 		gtk_object_unref (GTK_OBJECT (sheet_view));
 	}
 	g_list_free (sheet->sheet_views);
+	sheet->sheet_views = NULL;
 	g_list_free (sheet->comment_list);
+	sheet->comment_list = NULL;
 	
-	g_hash_table_foreach (sheet->cell_hash, cell_hash_free_key, NULL);
-	g_hash_table_destroy (sheet->cell_hash);
-
 	sheet_destroy_columns_and_rows (sheet);
 	sheet_destroy_styles (sheet);
 
+	g_hash_table_foreach (sheet->cell_hash, cell_hash_free_key, NULL);
+	g_hash_table_destroy (sheet->cell_hash);
+
 	sheet->signature = 0;
 	g_free (sheet);
 }
@@ -3420,3 +3419,41 @@ sheet_lookup_by_name (Sheet *base, char *name)
 
 	return NULL;
 }
+
+#if 0
+void
+sheet_insert_object (Sheet *sheet, char *repoid)
+{
+	GnomeClientSite *client_site;
+	GnomeObject *object_server;
+	GList *l;
+	
+	g_return_if_fail (sheet != NULL);
+	g_return_if_fail (IS_SHEET (sheet));
+	g_return_if_fail (repoid != NULL);
+
+	object_server = gnome_object_activate_with_repo_id (NULL, repoid, 0, NULL);
+	if (!object_server){
+		char *msg;
+
+		msg = g_strdup_printf (_("I was not able to activate object %s"), repoid);
+		
+		gnumeric_notice (sheet->workbook, GNOME_MESSAGE_BOX_ERROR, msg);
+		g_free (msg);
+		return;
+	}
+	
+	client_site = gnome_client_site_new (sheet->workbook->gnome_container);
+	gnome_container_add (sheet->workbook->gnome_container, GNOME_OBJECT (client_site));
+
+	if (!gnome_client_site_bind_component (client_site, object_server)){
+		gnumeric_notice (sheet->workbook, GNOME_MESSAGE_BOX_ERROR,
+				 _("I was unable to the bind object"));
+		gtk_object_unref (GTK_OBJECT (object_server));
+		gtk_object_unref (GTK_OBJECT (client_site));
+		return;
+	}
+	
+}
+
+#endif
