@@ -1,6 +1,12 @@
 #ifndef GNUMERIC_SHEET_H
 #define GNUMERIC_SHEET_H
 
+#ifdef ENABLE_BONOBO
+#    include <bonobo/gnome-container.h>
+#endif
+
+#include "solver.h"
+
 #define SHEET_MAX_ROWS (16 * 1024)
 #define SHEET_MAX_COLS 256
 
@@ -69,6 +75,12 @@ typedef struct {
 	void       *clipboard_paste_callback_data;
 
 	void       *toolbar;
+
+#ifdef ENABLE_BONOBO
+	/* A GnomeContainer */
+	GnomeContainer *gnome_container;
+	
+#endif
 } Workbook;
 
 typedef struct {
@@ -78,11 +90,19 @@ typedef struct {
 } SheetSelection;
 
 typedef enum {
+	/* Normal editing mode of the Sheet */
 	SHEET_MODE_SHEET,
+
+	/* Drawing object creation */
 	SHEET_MODE_CREATE_LINE,
 	SHEET_MODE_CREATE_BOX,
 	SHEET_MODE_CREATE_OVAL,
 	SHEET_MODE_CREATE_ARROW,
+
+	/* Selection for the region for a Graphics object */
+	SHEET_MODE_CREATE_GRAPHIC,
+
+	/* Object is selected */
 	SHEET_MODE_OBJECT_SELECTED,
 } SheetModeType;
 
@@ -136,6 +156,9 @@ typedef struct {
 	struct {
 		SheetSelection *current;
 	} walk_info;
+
+        /* Solver parameters */
+        SolverParameters solver_parameters;
 } Sheet;
 
 #define SHEET_SIGNATURE 0x12349876
@@ -147,7 +170,7 @@ typedef  void (*sheet_col_row_callback)(Sheet *sheet, ColRowInfo *ci,
 typedef  int (*sheet_cell_foreach_callback)(Sheet *sheet, int col, int row,
 					    Cell *cell, void *user_data);
 
-Sheet      *sheet_new                  	 (Workbook *wb, char *name);
+Sheet      *sheet_new                  	 (Workbook *wb, const char *name);
 void        sheet_rename                 (Sheet *sheet, const char *new_name);
 void        sheet_destroy              	 (Sheet *sheet);
 void        sheet_foreach_col          	 (Sheet *sheet,
@@ -353,6 +376,11 @@ char        *cellref_name                 (CellRef *cell_ref,
 					   int eval_col,
 					   int eval_row);
 
+/*
+ * Sheet, Bobobo objects
+ */
+void sheet_insert_object (Sheet *sheet, char *repoid);
+
 /*
  * Workbook
  */
@@ -375,7 +403,7 @@ void        workbook_auto_expr_label_set (Workbook *wb, char *text);
 void        workbook_next_generation     (Workbook *wb);
 void        workbook_set_region_status   (Workbook *wb, char *str);
 int         workbook_parse_and_jump      (Workbook *wb, char *text);
-Sheet      *workbook_sheet_lookup        (Workbook *wb, char *sheet_name);
+Sheet      *workbook_sheet_lookup        (Workbook *wb, const char *sheet_name);
 void        workbook_mark_clean          (Workbook *wb);
 gboolean    workbook_rename_sheet        (Workbook *wb,
 					  const char *old_name,
