@@ -7,6 +7,8 @@ void      float_get_from_range   (char *start, char *end, float_t *t);
 
 char      *cell_name             (int col, int row);
 int       parse_cell_name        (char *cell_str, int *col, int *row);
+GSList    *parse_cell_name_list  (void *sheet, char *cell_name_str,
+				  int *eflag);
 char      *col_name              (int col);
 int       col_from_name          (char *cell_str);
 
