@@ -172,6 +172,15 @@ italic_cmd (GtkWidget *widget, Workbook *wb)
 		change_selection_font (wb, 3, italic_names);
 }
 
+static void
+create_graphic_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet;
+
+	sheet = workbook_get_current_sheet (wb);
+	sheet_set_mode_type (sheet, SHEET_MODE_CREATE_GRAPHIC);
+}
+
 static void
 create_line_cmd (GtkWidget *widget, Workbook *wb)
 {
@@ -301,7 +310,7 @@ cb_sheet_check_dirty (gpointer key, gpointer value, gpointer user_data)
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(d)->vbox), l, TRUE, TRUE, 0);
 
 	gtk_window_set_position (GTK_WINDOW (d), GTK_WIN_POS_MOUSE);
-	button = gnome_dialog_run (GNOME_DIALOG (d));
+	button = gnome_dialog_run_and_close (GNOME_DIALOG (d));
 
 	switch (button){
 		/* YES */
@@ -322,8 +331,6 @@ cb_sheet_check_dirty (gpointer key, gpointer value, gpointer user_data)
 		break;
 
 	} 
-
-	gtk_widget_destroy (d);
 }
 
 static int
@@ -410,7 +417,8 @@ cut_cmd (GtkWidget *widget, Workbook *wb)
 		sheet_selection_cut (sheet);
 	else {
 		if (sheet->current_object){
-			sheet_object_destroy (sheet->current_object);
+			gtk_object_unref (GTK_OBJECT (sheet->current_object));
+			sheet->current_object = NULL;
 			sheet_set_mode_type (sheet, SHEET_MODE_SHEET);
 		} else
 			printf ("no object selected\n");
@@ -635,6 +643,23 @@ workbook_edit_comment (GtkWidget *widget, Workbook *wb)
 	dialog_cell_comment (wb, cell);
 }
 
+static void
+goal_seek_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet;
+	sheet = workbook_get_current_sheet (wb) ;
+	dialog_goal_seek (wb, sheet) ;
+}
+
+static void
+solver_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet;
+
+	sheet = workbook_get_current_sheet (wb);
+	dialog_solver (wb, sheet);
+}
+
 static void
 about_cmd (GtkWidget *widget, Workbook *wb)
 {
@@ -665,6 +690,17 @@ filenames_dropped (GtkWidget * widget,
 	}
 }
 
+#ifdef ENABLE_BONOBO
+static void
+insert_object_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	char *repoid = "IDL:Sample/server:1.0";
+
+/*	sheet_insert_object (sheet, repoid); */
+}
+#endif
+
 /* File menu */
 
 static GnomeUIInfo workbook_menu_file [] = {
@@ -724,6 +760,14 @@ static GnomeUIInfo workbook_menu_edit [] = {
 
 	GNOMEUIINFO_SEPARATOR,
 
+#ifdef ENABLE_BONOBO
+	GNOMEUIINFO_ITEM_NONE(N_("Insert object..."),
+			      N_("Inserts a Bonobo object"),
+			      insert_object_cmd),
+#endif
+
+	GNOMEUIINFO_SEPARATOR,
+
 #if 0
 	{ GNOME_APP_UI_ITEM, N_("_Define cell names"), NULL, define_cell_cmd },
 	GNOMEUIINFO_SEPARATOR,
@@ -811,6 +855,13 @@ static GnomeUIInfo workbook_menu_format [] = {
 	GNOMEUIINFO_END
 };
 
+/* Tools menu */
+static GnomeUIInfo workbook_menu_tools [] = {
+	{ GNOME_APP_UI_ITEM, N_("_Goal Seek..."), NULL, goal_seek_cmd },
+	{ GNOME_APP_UI_ITEM, N_("_Solver..."),    NULL, solver_cmd },
+	GNOMEUIINFO_END
+};
+
 static GnomeUIInfo workbook_menu_help [] = {
         GNOMEUIINFO_MENU_ABOUT_ITEM(about_cmd, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -824,6 +875,9 @@ static GnomeUIInfo workbook_menu [] = {
 	GNOMEUIINFO_MENU_VIEW_TREE(workbook_menu_view),
 	{ GNOME_APP_UI_SUBTREE, N_("_Insert"), NULL, workbook_menu_insert },
 	{ GNOME_APP_UI_SUBTREE, N_("F_ormat"), NULL, workbook_menu_format },
+#if UNTIL_FINISHED
+	{ GNOME_APP_UI_SUBTREE, N_("T_ools"), NULL, workbook_menu_tools },
+#endif
 	GNOMEUIINFO_MENU_HELP_TREE(workbook_menu_help),
 	GNOMEUIINFO_END
 };
@@ -875,6 +929,11 @@ static GnomeUIInfo workbook_toolbar [] = {
 	
 	GNOMEUIINFO_SEPARATOR,
 
+#ifdef ENABLE_BONOBO
+	GNOMEUIINFO_ITEM_DATA (
+		N_("Graphic"), N_("Creates a graphic in the spreadsheet"),
+		create_graphic_cmd, NULL, graphic_xpm),
+#endif
 	GNOMEUIINFO_ITEM_DATA (
 		N_("Line"), N_("Creates a line object"),
 		create_line_cmd, NULL, line_xpm),
@@ -1169,28 +1228,26 @@ change_auto_expr (GtkWidget *item, Workbook *wb)
 static void
 change_auto_expr_menu (GtkWidget *widget, GdkEventButton *event, Workbook *wb)
 {
-	static GtkWidget *menu;
-
-	if (!menu){
-		GtkWidget *item;
-		int i;
+	GtkWidget *menu;
+	GtkWidget *item;
+	int i;
 		
-		menu = gtk_menu_new ();
+	menu = gtk_menu_new ();
 
-		for (i = 0; quick_compute_routines [i].displayed_name; i++){
-			item = gtk_menu_item_new_with_label (
-				_(quick_compute_routines [i].displayed_name));
-			gtk_menu_append (GTK_MENU (menu), item);
-			gtk_widget_show (item);
-			gtk_signal_connect (GTK_OBJECT (item), "activate",
-					    GTK_SIGNAL_FUNC(change_auto_expr), wb);
-			gtk_object_set_data (GTK_OBJECT (item), "expr",
-					     quick_compute_routines [i].function);
-			gtk_object_set_data (GTK_OBJECT (item), "name",
-					     _(quick_compute_routines [i].displayed_name));
-		}
+	for (i = 0; quick_compute_routines [i].displayed_name; i++){
+		item = gtk_menu_item_new_with_label (
+			_(quick_compute_routines [i].displayed_name));
+		gtk_menu_append (GTK_MENU (menu), item);
+		gtk_widget_show (item);
+		gtk_signal_connect (GTK_OBJECT (item), "activate",
+				    GTK_SIGNAL_FUNC(change_auto_expr), wb);
+		gtk_object_set_data (GTK_OBJECT (item), "expr",
+				     quick_compute_routines [i].function);
+		gtk_object_set_data (GTK_OBJECT (item), "name",
+				     _(quick_compute_routines [i].displayed_name));
 	}
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 0, NULL, event->button, event->time);
+
+	gnumeric_popup_menu (GTK_MENU (menu), event);
 }
 
 /*
@@ -1384,6 +1441,10 @@ workbook_new (void)
 	workbook_list = g_list_prepend (workbook_list, wb);
 	
 	gtk_widget_show_all (wb->table);
+
+#ifdef ENABLE_BONOBO
+	wb->gnome_container = GNOME_CONTAINER (gnome_container_new ());
+#endif
 	return wb;
 }
 
@@ -1777,7 +1838,7 @@ workbook_detach_sheet (Workbook *wb, Sheet *sheet)
  * was not found.
  */
 Sheet *
-workbook_sheet_lookup (Workbook *wb, char *sheet_name)
+workbook_sheet_lookup (Workbook *wb, const char *sheet_name)
 {
 	Sheet *sheet;
 	
