@@ -3,7 +3,7 @@
  *
  * Daniel Veillard <Daniel.Veillard@w3.org>
  *
- * $Id: xml-io.c,v 1.36 1999/04/07 03:45:58 unammx Exp $
+ * $Id: xml-io.c,v 1.40 1999/04/15 19:24:46 unammx Exp $
  */
 
 #include <config.h>
@@ -14,6 +14,7 @@
 #include "gnome-xml/parser.h"
 #include "color.h"
 #include "sheet-object.h"
+#include "sheet-object-graphic.h"
 #include "xml-io.h"
 #include "file.h"
 
@@ -205,7 +206,7 @@ xmlGetCoordinates (xmlNodePtr node, const char *name,
 	res = sscanf (ret, "(%lf %lf)(%lf %lf)", x1, y1, x2, y2);
 	free(ret);
 	
-	if (res == 2) 
+	if (res == 4) 
 		return 1;
 
 	return 0;
@@ -1102,30 +1103,34 @@ readXmlColRowInfo (parseXmlContextPtr ctxt,
 static xmlNodePtr
 writeXmlObject (parseXmlContextPtr ctxt, SheetObject *object)
 {
+	SheetObjectGraphic *sog = SHEET_OBJECT_GRAPHIC (object);
 	xmlNodePtr cur = NULL;
 
-	switch (object->type){
+	switch (sog->type){
 	case SHEET_OBJECT_RECTANGLE:{
-			SheetFilledObject *fo = (SheetFilledObject *) object;
+			SheetObjectFilled *sof = SHEET_OBJECT_FILLED (object);
 
 			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Rectangle", NULL);
-			if (fo->fill_color != NULL)
-				xmlSetStringValue (cur, "FillColor", fo->fill_color);
-			xmlSetIntValue (cur, "Pattern", fo->pattern);
+			if (sof->fill_color != NULL)
+				xmlSetStringValue (cur, "FillColor", sof->fill_color);
+			xmlSetIntValue (cur, "Pattern", sof->pattern);
 			break;
 		}
+
 	case SHEET_OBJECT_ELLIPSE:{
-			SheetFilledObject *fo = (SheetFilledObject *) object;
+			SheetObjectFilled *sof = SHEET_OBJECT_FILLED (object);
 
 			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Ellipse", NULL);
-			if (fo->fill_color != NULL)
-				xmlSetStringValue (cur, "FillColor", fo->fill_color);
-			xmlSetIntValue (cur, "Pattern", fo->pattern);
+			if (sof->fill_color != NULL)
+				xmlSetStringValue (cur, "FillColor", sof->fill_color);
+			xmlSetIntValue (cur, "Pattern", sof->pattern);
 			break;
 		}
+
 	case SHEET_OBJECT_ARROW:
 		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Arrow", NULL);
 		break;
+
 	case SHEET_OBJECT_LINE:
 		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Line", NULL);
 		break;
@@ -1133,9 +1138,10 @@ writeXmlObject (parseXmlContextPtr ctxt, SheetObject *object)
 	if (cur == NULL)
 		return NULL;
 	
-	xmlSetGnomeCanvasPoints (cur, "Points", object->points);
-	xmlSetIntValue (cur, "Width", object->width);
-	xmlSetStringValue (cur, "Color", object->color);
+	xmlSetGnomeCanvasPoints (cur, "Points", object->bbox_points);
+	xmlSetIntValue (cur, "Width", sog->width);
+	xmlSetStringValue (cur, "Color", sog->color);
+
 	return cur;
 }
 
@@ -1146,7 +1152,7 @@ static SheetObject *
 readXmlObject (parseXmlContextPtr ctxt, xmlNodePtr tree)
 {
 	SheetObject *ret;
-	SheetFilledObject *fo;
+	SheetObjectFilled *sof;
 	char *color;
 	char *fill_color;
 	int type;
@@ -1179,8 +1185,8 @@ readXmlObject (parseXmlContextPtr ctxt, xmlNodePtr tree)
 		ret = sheet_object_create_filled (ctxt->sheet, type,
 			       x1, y1, x2, y2, fill_color, color, width);
 		if (ret != NULL){
-			fo = (SheetFilledObject *) ret;
-			fo->pattern = pattern;
+			sof = SHEET_OBJECT_FILLED (ret);
+			sof->pattern = pattern;
 		}
 	} else {
 		ret = sheet_object_create_line (ctxt->sheet, type,
@@ -1623,7 +1629,7 @@ xml_probe (const char *filename)
 		return FALSE;
 
 	gmr = xmlSearchNsByHref (res, res->root, "http://www.gnome.org/gnumeric/");
-	if (strcmp (res->root->name, "Workbook") || (gmr == NULL)){
+	if (res->root->name == NULL || strcmp (res->root->name, "Workbook") || (gmr == NULL)){
 		xmlFreeDoc (res);
 		return FALSE;
 	}
